# =============
# QPACE/TNWTEST
# =============
# $Id: endpoint.pm 3149 2010-02-03 18:17:22Z pleiter $
#---------------------------------------------------------------------------------------------------

package Endpoint;

use strict;

use lib("$ENV{QRUNDIR}/pm");
use node;
use intf;


#---------------------------------------------------------------------------------------------------
# Constructor
#---------------------------------------------------------------------------------------------------

sub new
{
  my $class = shift;
  my @argv  = @_;

  my $self           = {};
  $self->{node}      = undef;
  $self->{intf}      = undef;
  $self->{status}    = {};
  $self->{state}     = undef;	# exerciser state
  $self->{completed} = 0;	# true if exerciser completed on SPU
  $self->{mismatch}  = [];

  if (scalar @argv == 4) {	# expect: bp, nc, phy, pr
    $self->{node} = Node->new($argv[0], $argv[1]);
    $self->{intf} = Intf->new($argv[2], $argv[3]);
  }
  elsif (scalar @argv == 3 && ref($argv[0]) eq "Node") { # expect: node, phy, pr
    $self->{node} = $argv[0];
    $self->{intf} = Intf->new($argv[1], $argv[2]);
  }
  elsif (scalar @argv == 3) {	# expect: bp, nc, phy
    $self->{node} = Node->new($argv[0], $argv[1]);
    $self->{intf} = Intf->new($argv[2]);
  }
  elsif (scalar @argv == 2 && ref($argv[0]) eq "Node") { # expect: node, phy
    $self->{node} = $argv[0];
    $self->{intf} = Intf->new($argv[1]);
  }

  unless (defined $self->{node} && defined $self->{intf}) {
    die "Contructor for class Endpoint failed";
  }

  bless ($self, $class);
  return $self;
}


#---------------------------------------------------------------------------------------------------
# Member functions
#---------------------------------------------------------------------------------------------------

sub as_string_pr
{
  my $self = shift;

  my $str = undef;
  if (defined $self->{node}) {
    if (defined $self->{intf}) {
      $str = sprintf "%s::%s", $self->{node}->hostname, $self->{intf}->as_string;
    }
    else {
      $str = $self->{node}->hostname;
    }
  }

  return $str;
}

sub as_string
{
  my $self = shift;

  my $str = undef;
  if (defined $self->{node}) {
    if (defined $self->{intf}) {
      $str = sprintf "%s::%d", $self->{node}->hostname, $self->{intf}->{phy};
    }
  }

  return $str;
}


sub hostname
{
  my $self = shift;
  return $self->{node}->hostname;
}

1;
