#!/usr/bin/perl

use strict;
use FileHandle;

use constant TIMER_WIDTH	 => 10;
use constant ADDR_WIDTH		 => 17;
use constant DATA_WIDTH		 => 32;
use constant CFUCMD_ADDRHI_WIDTH =>  5;
use constant CFUCMD_ADDRLO_WIDTH => 17;

use constant PIPELINE0_DEPTH	=> 23;
use constant PIPELINE1_DEPTH	=>  3;
use constant PIPELINE2_DEPTH	=>  3;
use constant PIPELINE3_DEPTH	=>  6;

use constant PIPELINE4_DEPTH	=> 16;
use constant PIPELINE5_DEPTH	=> 16;
use constant PIPELINE6_DEPTH	=> 16;
use constant PIPELINE9_DEPTH	=> 16;

my %nIwcReg = (
	"iwcc8"		=> 32,
	"mgbtrace0"	=> 98,
);

my $mode = "mgbtrace0";

die "nIwcReg not defined for mode \"$mode\"\n" unless defined $nIwcReg{$mode};


foreach my $file (@ARGV) {
  my $fh = new FileHandle;

  if ($file =~ /\.bz2$/) {
    $fh->open("bzcat $file |");
  }
  else {
    $fh->open("< $file");
  }
  die "Failed to open \"$file\" for reading\n" unless defined $fh;

  my %s;
  my $ns = 0;
  while (getSnapShot(\%s, $fh)) {
    timerOverflowCorrect(\%s);
    sortEventLst(\%s);
    prSnapShot(\%s, $ns++);
  }

  $fh->close();
}


#-------------------------------------------------------------------------------
# Parser
#-------------------------------------------------------------------------------

sub getSnapShot($$)
{
  my ($s, $fh) = @_;

  undef %{$s};

  $s->{ckTrigger} = undef;
  $s->{ckData}    = undef;
  $s->{ckAddr}    = undef;
  $s->{eventLst}  = [];

  my @fill = (0x0, 0x0, 0x0, 0x0);

  # temporary work-around (fill3 not written to DCR reg)
#  $fill[3] = 0xff;

  my $filled = 0;
  while (<$fh>) {
    if (/\s10\.98\.1(\d\d)\.1(\d\d)\s/) {
      $s->{bp} = $1;
      $s->{nc} = $2;
    }
    if (/\sIWC\[(\d+)\]=0x([0-9a-f]+)/i) {
      my ($reg, $val) = ($1, hex($2));

      if ($reg == 0) {
        $s->{lineMin} = $.;
      }

      if ($reg+1 >= $nIwcReg{$mode}) {
        $s->{lineMax} = $.;
        $filled = 1;
      }

      my $regMin = 0;
      my $regMax = 0;
      if ($regMin <= $reg && $reg <= $regMax) {
        my $p = 0;		# position
        my $w = 1;		# width
        $s->{ckTrigger} = ($val >> $p) & (2**$w - 1);

        $p += $w;
        $w = PIPELINE1_DEPTH;
        $fill[1] = ($val >> $p) & (2**$w - 1);
#printf "DEBUG val=0x%08x fill[1]=0x%02x\n", $val, $fill[1];

        $p += $w;
        $w = PIPELINE2_DEPTH;
        $fill[2] = ($val >> $p) & (2**$w - 1);

        $p += $w + 1;
        $w = PIPELINE0_DEPTH;
        $fill[0] = ($val >> $p) & (2**$w - 1);

        #printf "DEBUG: IWC[0]=0x%08x trigger=%d fill[0]=0x%04x " .
        #       "fill[1]=0x%04x fill[2]=0x%04x\n",
        #       $val, $s->{trigger}, @fill[0..2];
      }

      $regMin = $regMax + 1;
      $regMax += 1;
      if ($regMin <= $reg && $reg <= $regMax) {
        my $p = 0;		# position
        my $w = ADDR_WIDTH;	# width
        $s->{ckAddr} = ($val >> $p) & (2**$w - 1);

        $p += $w + 3;
        $w = PIPELINE3_DEPTH;
        $fill[3] = ($val >> $p) & (2**$w - 1);

        #printf "DEBUG IWC[%d]=0x%08x addr=0x%08x\n", $reg, $val, $s->{addr};
      }

      $regMin = $regMax + 1;
      $regMax += 1;
      if ($regMin <= $reg && $reg <= $regMax) {
        my $p = 0;
        my $w = DATA_WIDTH;
        $s->{ckData} = ($val >> $p) & (2**$w - 1);
        #printf "DEBUG IWC[%d]=0x%08x data=0x%08x\n", $reg, $val, $s->{data};
      }

      $regMin = $regMax + 1;
      $regMax += PIPELINE2_DEPTH;
      if ($regMin <= $reg && $reg <= $regMax) {
        my $i = $reg - $regMin;

        if ($fill[1] & (0x1 << $i)) {			# pipeline 1
          push @{$s->{eventLst}}, {};
          my $event = $s->{eventLst}->[$#{$s->{eventLst}}];

          $event->{type} = "REQ";

          my $p = 0;
          my $w = TIMER_WIDTH;
          $event->{timer} = ($val >> $p) & (2**$w - 1);

          $p += $w;
          $w = 1;
          $event->{size} = ($val >> $p) & (2**$w - 1) ? 256 : 128;

          $p += $w;
          $w = 3;
          $event->{link} = ($val >> $p) & (2**$w - 1);

          $p += $w;
          $w = 3;
          $event->{ch} = ($val >> $p) & (2**$w - 1);

          $p += $w;
          $w = 3;
          $event->{a128} = ($val >> $p) & (2**$w - 1);
#printf "p=$p, w=$w, a128=%d\n", (($val >> $p) & (2**$w - 1));

#	  printf "DEBUG IWC[%d]=0x%08x size=%d link=%d ch=%d a128=%d\n",
#                 $reg, $val, $event->{size}, $event->{link}, $event->{ch},
#                 $event->{a128};
        }


        if ($fill[2] & (0x1 << $i)) {			# pipeline 2
          push @{$s->{eventLst}}, {};
          my $event = $s->{eventLst}->[$#{$s->{eventLst}}];

          $event->{type} = "LAST";

          my $p = TIMER_WIDTH + 1 + 3 + 3 + 3;
          my $w = TIMER_WIDTH;
          $event->{timer} = ($val >> $p) & (2**$w - 1);
        }
      }

      $regMin = $regMax + 1;
      $regMax += PIPELINE3_DEPTH/2;
      if ($regMin <= $reg && $reg <= $regMax) {
        my $k = 2. * ($reg - $regMin);
        foreach my $l (0..1) {
          my $i = $k + $l;
          if ($fill[3] & (0x1 << $i)) {			# pipeline 3
            push @{$s->{eventLst}}, {};
            my $event = $s->{eventLst}->[$#{$s->{eventLst}}];

            $event->{type} = "FULL";

            my $p = $l * 16;
            my $w = TIMER_WIDTH;
            $event->{timer} = ($val >> $p) & (2**$w - 1);

            $p += $w + 2;
            $w = 2;
            $event->{full} = ($val >> $p) & (2**$w - 1);
          }
        }
      }

      $regMin = $regMax + 1;
      $regMax += PIPELINE0_DEPTH;
      if ($regMin <= $reg && $reg <= $regMax) {
        my $i = $reg - $regMin;

        if ($fill[0] & (0x1 << $i)) {			# pipeline 0
          push @{$s->{eventLst}}, {};
          my $event = $s->{eventLst}->[$#{$s->{eventLst}}];

          $event->{type} = "RE";

          my $p = 0;
          my $w = TIMER_WIDTH;
          $event->{timer} = ($val >> $p) & (2**$w - 1);
#printf "$p $w 0x%08x 0x%08x\n", $event->{timer}, (2**$w - 1);

          $p += $w;
          $w = 32 - TIMER_WIDTH;
          $event->{data} = ($val >> $p) & (2**$w - 1);
#printf "%s", $_;
#printf "0x%08x %d 0x%08x\n", $event->{timer}, $event->{timer}, $event->{data};
        }
      }

      # Filling pipe 5 and 4
      $regMin = $regMax + 1;
      $regMax += 1;
      if ($regMin <= $reg && $reg <= $regMax) {
      }

      # Filling pipe 9 and 6
      $regMin = $regMax + 1;
      $regMax += 1;
      if ($regMin <= $reg && $reg <= $regMax) {
      }

      # Pipeline 4
      $regMin = $regMax + 1;
      $regMax += PIPELINE4_DEPTH;
      if ($regMin <= $reg && $reg <= $regMax) {
        push @{$s->{eventLst}}, {};
        my $event = $s->{eventLst}->[$#{$s->{eventLst}}];

        $event->{type} = "F2BDATA";

        my $p = 0;
        my $w = TIMER_WIDTH;
        $event->{timer} = ($val >> $p) & (2**$w - 1);

        $p += $w;
        $w = 32 - TIMER_WIDTH;
        $event->{data} = ($val >> $p) & (2**$w - 1);
        #printf "DEBUG (%d) IWC[%d]=0x%08x time=0x%03x mgbData=0x%08x\n",
        #       $., $reg, $val, $event->{timer}, $event->{data};
      }

      # Pipeline 5
      $regMin = $regMax + 1;
      $regMax += PIPELINE5_DEPTH;
      if ($regMin <= $reg && $reg <= $regMax) {
        push @{$s->{eventLst}}, {};
        my $event = $s->{eventLst}->[$#{$s->{eventLst}}];

        $event->{type} = "CFUCMD";

        my $p = 0;
        my $w = TIMER_WIDTH;
        $event->{timer} = ($val >> $p) & (2**$w - 1);

        $p += $w;
        $w = CFUCMD_ADDRLO_WIDTH;
        $event->{addrLo} = ($val >> $p) & (2**$w - 1);

        $p += $w;
        $w = CFUCMD_ADDRHI_WIDTH;
        $event->{addrHi} = ($val >> $p) & (2**$w - 1);
      }

      # Pipeline 6
      $regMin = $regMax + 1;
      $regMax += PIPELINE6_DEPTH;
      if ($regMin <= $reg && $reg <= $regMax) {
        push @{$s->{eventLst}}, {};
        my $event = $s->{eventLst}->[$#{$s->{eventLst}}];

        $event->{type} = "WDBADDR";

        my $p = 0;
        my $w = TIMER_WIDTH;
        $event->{timer} = ($val >> $p) & (2**$w - 1);

        $p = 16;
        $w = 8;
        $event->{addr} = ($val >> $p) & (2**$w - 1);
      }

      # Pipeline 9
      $regMin = $regMax + 1;
      $regMax += PIPELINE9_DEPTH;
      if ($regMin <= $reg && $reg <= $regMax) {
        push @{$s->{eventLst}}, {};
        my $event = $s->{eventLst}->[$#{$s->{eventLst}}];

        $event->{type} = "F2BSTAT";

        my $p = 0;
        my $w = TIMER_WIDTH;
        $event->{timer} = ($val >> $p) & (2**$w - 1);

        $p = 16;
        $w = 2;
        $event->{val} = ($val >> $p) & (2**$w - 1);
      }
    }

    last if ($filled == 1);
  }

  # Oldest entries (end of pipeline) first
  if ($filled) {
    @{$s->{eventLst}} = reverse @{$s->{eventLst}};
  }

  return $filled;
}


#-------------------------------------------------------------------------------
# Printer
#-------------------------------------------------------------------------------

sub prSnapShot($$)
{
  my ($s, $ns) = @_;

  printf ">>> SNAPSHOT #%d\n", $ns;

  printf "    node      = nc-%02d-%02d\n", $s->{bp}, $s->{nc};
  printf "    lineMin   = %d\n", $s->{lineMin} if defined $s->{lineMin};
  printf "    lineMax   = %d\n", $s->{lineMax} if defined $s->{lineMax};
  printf "    ckTrigger = %d\n", $s->{ckTrigger} if defined $s->{ckTrigger};
  if (defined $s->{ckTrigger} && $s->{ckTrigger}) {
    printf "    ckData    = 0x%08x\n", $s->{ckData} if defined $s->{ckData};
    printf "    ckAddr    = 0x%08x\n", $s->{ckAddr} if defined $s->{ckAddr};
  }

  foreach my $event (@{$s->{eventLst}}) {
    my ($op, $str) = ("", "");
    for ($event->{type}) {
      ($_ eq "REQ") && do {
        $str = sprintf "size=%d lnk=%d ch=%d a128=%d", $event->{size},
                       $event->{link}, $event->{ch}, $event->{a128};
        last;
      };

      ($_ eq "LAST") && do {
        last;
      };

      ($_ eq "FULL") && do {
        $str .= "s1full " if ($event->{full} & 0x1);
        $str .= "s2full " if ($event->{full} & 0x2);
        last;
      };

      ($_ eq "RE" || $_ eq "F2BDATA") && do {
        my $data  = $event->{data};
        my $a16   = ($data >>  0) & 0x7;
        my $a128  = ($data >>  3) & 0xf;
        my $iMsg  = ($data >>  7) & 0x7;
        my $iter  = ($data >> 10) & 0x7;
        my $ch    = ($data >> 13) & 0x7;
        my $lnk   = ($data >> 16) & 0x7;
        my $ncPos = ($data >> 19) & 0x7;
        $str = sprintf "0x%08x  ncPos=%d lnk=%d ch=%d iter=%d iMsg=%d " .
                       "a128=%d a16=%d", $data, $ncPos, $lnk, $ch, $iter,
                       $iMsg, $a128, $a16;
        last;
      };

      ($_ eq "CFUCMD") && do {
        my $addrHi = $event->{addrHi};
        my $addrLo = $event->{addrLo};
        my $a128   = ($addrLo >>  0) & 0x7ff;
        my $ch     = ($addrLo >> 11) & 0x007;
        my $lnk    = ($addrLo >> 14) & 0x007;
        $str = sprintf "addrHi=0x%02x addrLo=0x%08x " .
                       "base=0x%02x0_00000000 lnk=%d ch=%d a128=%d",
                       $addrHi, $addrLo, $addrHi, $lnk, $ch, $a128;
        last;
      };

      ($_ eq "WDBADDR") && do {
        my $addr = $event->{addr};
        $str = sprintf "addr=0x%02x", $addr;
        last;
      };

      ($_ eq "F2BSTAT") && do {
        my $val = $event->{val};
        my $msg = "";
        $msg .= " PTI" if ($val & 0x1);
        $msg .= " DERR" if ($val & 0x2);
        $str = sprintf "val=0x%02x%s", $val, $msg;
        last;
      };

      # default
      printf STDERR "Unkown event type \"%s\"\n", $event->{type};
      die;
    }

    printf "    %06d  %-7s  %s\n", $event->{timer}, $event->{type}, $str;
  }

  printf "<<<\n";
}


#-------------------------------------------------------------------------------
# Try to correct timer overflow
#-------------------------------------------------------------------------------

sub timerOverflowCorrect($)
{
  my ($s) = @_;

  my $nEvent = scalar @{$s->{eventLst}};

  foreach my $type ("REQ", "LAST", "FULL", "RE") {
    my $timerOld = undef;
    foreach my $i (0..$nEvent-1) {
      next if ($s->{eventLst}->[$i]->{type} ne $type);

      if (defined $timerOld && $timerOld > $s->{eventLst}->[$i]->{timer}) {
        $s->{eventLst}->[$i]->{timer} += 2**TIMER_WIDTH;
      }
      $timerOld = $s->{eventLst}->[$i]->{timer};
    }
  }
}


#-------------------------------------------------------------------------------
# Sort entries according to time
#-------------------------------------------------------------------------------

sub myCmp($$)
{
  my ($a, $b) = @_;

  # When comparing events of same time do not change order!
  return 0 if ($a->{type} eq $b->{type});

  return ($a->{timer} <=> $b->{timer});
}

sub sortEventLst($)
{
  my ($s) = @_;

  @{$s->{eventLst}} = sort { myCmp($a, $b) } @{$s->{eventLst}};
}
