#!/usr/bin/perl -w
#
# Extract RXH and /E/ counters from tnwTestParser output files
#
#---------------------------------------------------------------------------------------------------

use strict;
use endpoint;

my %epLst;


#---------------------------------------------------------------------------------------------------
# Parse input files
#
my $ep;
foreach my $file (@ARGV) {
  my $flag = 0;

  open(IN, "< $file") || die "Failed to open \"$file\" for reading\n";
  while (<IN>) {
    chomp;

    /LINK STATUS/		&& do { $flag = 1; next; };
    /STATUS/		&& do { $flag = 0; next; };

    next unless $flag;

    if (/nc-(\d+)-(\d+)::(\d)([pr]).*nc-(\d+)-(\d+)::(\d)([pr])/) {
      $ep = Endpoint->new($1, $2, $3, $4);
      unless (defined $epLst{$ep->as_string}) {
        $epLst{$ep->as_string} = $ep;
        $epLst{$ep->as_string}->{status}->{RXHMAX} = 0;
        $epLst{$ep->as_string}->{status}->{EMAX} = 0;
      }
      next;
    }

    if (/RX_RXH = 0x([[:xdigit:]]+)/) {
      my $rxh = hex($1);
      foreach my $i (0..3) {
        my $val = ($rxh >> (8*$i)) & 0xff;
        if ($val > $epLst{$ep->as_string}->{status}->{RXHMAX}) {
          $epLst{$ep->as_string}->{status}->{RXHMAX} = $val;
        }
      }
      next;
    }

    if (/cnt..E.. = 0x([[:xdigit:]]+)/) {
      my $err = hex($1);
      if ($err > $epLst{$ep->as_string}->{status}->{EMAX}) {
        $epLst{$ep->as_string}->{status}->{EMAX} = $err;
      }
      next;
    }
  }

  close(IN);
}


#---------------------------------------------------------------------------------------------------
# Print statistics
#
printf "Maximum RXH and /E/ counters\n";
printf "----------------------------\n";
my @keyLst = reverse sort { $epLst{$a}->{status}->{EMAX} <=> $epLst{$b}->{status}->{EMAX} }
             keys %epLst;
foreach my $key (@keyLst) {
  printf "%10s 0x%02x 0x%04x\n",
         $key, $epLst{$key}->{status}->{RXHMAX}, $epLst{$key}->{status}->{EMAX};
}
printf "\n";
