#!/usr/bin/perl -w

use strict;

my $file = $ARGV[0];
my $flag = 0;
my $node = undef;
my $key  = undef;

my %stat;


#-------------------------------------------------------------------------------
# Parse input file
#
open(IN, "< $file") || die "Failed to open \"$file\" for reading\n";
while (<IN>) {
  /FLEXIO STATUS/	&& do { $flag = 1; next; };
  /STATUS/		&& do { $flag = 0; next; };
  /(nc-\d+-\d+)/	&& do { $node = $1; next; };
  /Soma retry counter/	&& do { $key  = "somaRetry"; next; };
  /Soma CRC counter/	&& do { $key  = "somaCrc"; next; };
  /GBIF retry counter/	&& do { $key  = "gbifRetry"; next; };
  /GBIF CRC counter/	&& do { $key  = "gbifCrc"; next; };

  next unless $flag;
  next unless defined $key;

  next if ($key =~ /gbif/ && /0xffffffff/);
  next if ($key =~ /gbif/ && /0x00000fb0/);
  next if ($key eq "gbifCrc" && /0xfffff000/);
  next if ($key eq "gbifCrc" && /0x04000c00/);

  if (/0x([[:xdigit:]]+)/) {
    $stat{$node} = {} unless defined $stat{$node};
    $stat{$node}->{$key} = [] unless defined $stat{$node}->{$key};
    push @{$stat{$node}->{$key}}, hex($1);
  }
}
close(IN);


#-------------------------------------------------------------------------------
# Print statistics
#
print "# node     somaRetry   gbifCrc   somaCrc gbifRetry\n";
for my $node (sort keys %stat) {
  my %d;
  for my $key ("somaRetry", "gbifCrc", "somaCrc", "gbifRetry") {
    $d{$key} = 0;
    if (defined $stat{$node}->{$key}) {
      $d{$key} = $stat{$node}->{$key}->[$#{$stat{$node}->{$key}}] -
                 $stat{$node}->{$key}->[0];
    }
  }

  printf "  %s  %8d  %8d  %8d  %8d\n",
         $node, $d{somaRetry}, $d{gbifCrc}, $d{somaCrc}, $d{gbifRetry};
}
