#!/usr/bin/perl -w

# Example:
# node=10.98.100.118 lnk=0 err iMsg=2 iter=10705529 i=63 0x00a9e4fc_0095e4fd_008de4fe_00a9e4ff 0x00a9e4fc_0095e4fd_008de4fe_00a9e4ff

use strict;

foreach my $str (@ARGV) {
  $str =~ s/0x//g;

  my @f = split /_/, $str;

  die "Parse error ($str)" if ($#f != 3);

  print ">>> $str\n";

  my ($lnk0, $ch0, $iMsg0, $msgSize0) = (undef, undef, undef, undef);
  my ($a1280, $a160) = (undef, undef);
  my ($iterHi, $iterLo0) = (0, undef);
  my $msg;
  foreach my $i (0..3) {

    my $lnk = (hex($f[$i]) >> 29) & 0x7;
    $msg = "";
    if (defined $lnk0) {
      $msg = "--> link number changed!" if ($lnk != $lnk0);
    }
    else {
      $lnk0 = $lnk;
    }
    if ($i == 3 || $msg ne "") {
      printf "    lnk[%d]    = %d\t%s\n", $i, $lnk, $msg;
    }

    my $ch  = (hex($f[$i]) >> 26) & 0x7;
    $msg = "";
    if (defined $ch0) {
      $msg = "--> channel number changed!" if ($ch != $ch0);
    }
    else {
      $ch0 = $ch;
    }
    if ($i == 3 || $msg ne "") {
      printf "    ch[%d]     = %d\t%s\n", $i, $ch, $msg;
    }

    my $iMsg  = (hex($f[$i]) >> 22) & 0xf;
    $msg = "";
    if (defined $iMsg0) {
      $msg = "--> iMsg changed!" if ($iMsg != $iMsg0);
    }
    else {
      $iMsg0 = $iMsg;
    }
    if ($i == 3 || $msg ne "") {
      printf "    iMsg[%d]   = %d\t%s\n", $i, $iMsg, $msg;
    }

    my $iterLo  = (hex($f[$i]) >> 10) & 0xff;
    $msg = "";
    if (defined $iterLo0) {
      $msg = "--> iterLo changed!" if ($iterLo != $iterLo0);
    }
    else {
      $iterLo0 = $iterLo;
    }
    if ($i == 3 || $msg ne "") {
      printf "    iterLo[%d] = %d\t%s\n", $i, $iterLo, $msg;
    }

    $iterHi |= ((hex($f[$i]) >> 18) & 0xf ) << (4*$i);
    if ($i == 3) {
      my $iter = ($iterHi << 8) | $iterLo;
      printf "    iterHi    = %d\n", $iterHi;
      printf "    iter      = %d\n", $iter;
    }

    my $a128  = (hex($f[$i]) >> 5) & 0x1f;
    $msg = "";
    if (defined $a1280) {
      $msg = "--> a128 changed!" if ($a128 != $a1280);
    }
    else {
      $a1280 = $a128;
    }
    if ($i == 3 || $msg ne "") {
      printf "    a128[%d]   = %d\t%s\n", $i, $a128, $msg;
    }

    my $a16  = (hex($f[$i]) >> 2) & 0x7;
    $msg = "";
    if (defined $a160) {
      $msg = "--> a16 changed!" if ($a16 != $a160);
    }
    else {
      $a160 = $a16;
    }
    if ($i == 3 || $msg ne "") {
      printf "    a16[%d]    = %d\t%s\n", $i, $a16, $msg;
    }

    my $a4  = (hex($f[$i]) >> 0) & 0x3;
    my $a   = 32*$a1280 + 4*$a160 + $a4;
    $msg = sprintf "--> a=%d, i=%d\t", $a, ($a/4);
    $msg .= ($a4 != $i) ?"--> a4 mismatch!" : "";
    printf "    a4[%d]     = %d\t%s\n", $i, $a4, $msg;
  }
}
