# =============
# QPACE/TNWTEST
# =============
# $Id: pattern5.pm 3149 2010-02-03 18:17:22Z pleiter $
#---------------------------------------------------------------------------------------------------

package Pattern5;

use strict;
use node;
use endpoint;


#---------------------------------------------------------------------------------------------------
# Constructor
#---------------------------------------------------------------------------------------------------

sub new
{
  my $class = shift;
  my $self  = {};

  $self->{val}    = [];
  $self->{errMsg} = [];
  $self->{tx}     = undef;	# transmitter end-point
  $self->{ch}     = undef;
  $self->{iter}   = undef;
  $self->{iMsg}   = undef;
  $self->{a128}   = [];
  $self->{a16}    = [];
  $self->{a4}     = [];

  die "Too many arguments" if (scalar @_ > 4);

  my ($phy, $phy0, $ch0, $iMsg) = (undef, undef, 0);
  my ($iterHi, $iterLo0) = (0, undef);
  my ($ncPos1, $ncPos10, $ncPos2) = (undef, undef, 0);

  foreach my $i (0..$#_) {
    my $val = hex($_[$i]);

    push @{$self->{val}}, $val;

    $phy = ($val >> 29) & 0x7;
    if (defined $phy0) {
      if ($phy0 != $phy) {
        push @{$self->{errMsg}},
             sprintf "link number changed %d -> %d", $phy0, $phy;
      }
    }
    else {
      $phy0 = $phy;
    }

    $self->{ch} = ($val >> 26) & 0x7;
    if (defined $ch0) {
      if ($ch0 != $self->{ch}) {
        push @{$self->{errMsg}},
             sprintf "channel number changed %d -> %d", $ch0, $self->{ch};
      }
    }
    else {
      $ch0 = $self->{ch};
    }

    my $a128 = ($val >> 21) & 0x1f;
    push @{$self->{a128}}, $a128;

    my $a16 = ($val >> 18) & 0x7;
    push @{$self->{a16}}, $a16;

    my $a4 = ($val >> 16) & 0x3;
    push @{$self->{a4}}, $a4;

    $self->{iMsg} |= (($val >> 15) & 0x1) << (1*$a4);

    my $iterLo  = ($val >> 7) & 0x7;
    if (defined $iterLo0) {
      if ($iterLo != $iterLo0) {
        push @{$self->{errMsg}},
             sprintf "iterLo changed %d -> %d", $iterLo0, $iterLo;
      }
    }
    else {
      $iterLo0 = $iterLo;
    }

    my $iterHi |= (($val >> 10) & 0x1f ) << (5*$a4);
    if ($i == $#_) {
      $self->{iter} = ($iterHi << 3) | $iterLo;
    }

    my $ncPos1 = ($val >> 0) & 0x3f;
    if (defined $ncPos10) {
      if ($ncPos1 != $ncPos10) {
        push @{$self->{errMsg}},
             sprintf "ncPos1 changed %d -> %d", $ncPos10, $ncPos1;
      }
    }
    else {
      $ncPos10 = $ncPos1;
    }

    $ncPos2 |= (($val >> 6) & 0x1) << (1*$a4);
    if ($i == $#_) {
      my $ncPos = (($ncPos1 <<  4) & 0x0300) |
                  (($ncPos1 <<  0) & 0x000f) |
                  (($ncPos2 <<  9) & 0x1c00) |
                  (($ncPos2 <<  4) & 0x0010);

      my $nc   = ($ncPos >> 0) & 0x1f;
      my $bp   = ($ncPos >> 8) & 0x1f;
      my $node = Node->new($bp, $nc);

      $self->{tx} = Endpoint->new($node, $phy);
    }
  }

  bless ($self, $class);
  return $self;
}


#---------------------------------------------------------------------------------------------------
# Translate intro string
#---------------------------------------------------------------------------------------------------

sub as_string
{
  my $self   = shift;
  my $prefix = shift;

  my $str = "";

  $str .= "${prefix}0x";
  foreach my $i (0..$#{$self->{val}}) {
    $str .= sprintf "%08x", $self->{val}->[$i];
    $str .= "_" if ($i < $#{$self->{val}});
  }
  $str .= "\n";

  $str .= sprintf "${prefix}tx    = %s\n", $self->{tx}->as_string();
  $str .= sprintf "${prefix}ch    = %d\n", $self->{ch}   if (defined $self->{ch});
  $str .= sprintf "${prefix}iter  = %d\n", $self->{iter} if (defined $self->{iter});
  $str .= sprintf "${prefix}iMsg  = %d\n", $self->{iMsg} if (defined $self->{iMsg});
  $str .= sprintf "${prefix}a128  = %d\n", $self->a128 if ($self->na128 > 0);
  $str .= sprintf "${prefix}a16   = %d\n", $self->a16  if ($self->na16 > 0);
  if ($self->na4 > 0) {
    $str .= "${prefix}a4    =";
    foreach my $i (0..$self->na4-1) {
      $str .= sprintf " %d", $self->a4($i);
    }
    $str .= "\n";
  }

  return $str;
}


#---------------------------------------------------------------------------------------------------
# Other member functions
#---------------------------------------------------------------------------------------------------

sub allZero
{
  my $self = shift;

  foreach (@{$self->{val}}) {
    return 0 if ($_);
  }

  return 1;
}


sub na128
{
  my $self = shift;
  return scalar @{$self->{a128}};
}


sub a128
{
  my $self = shift;
  my $i = shift;

  $i = defined $i ? $i : 0;

  return $self->{a128}->[$i];
}


sub na16
{
  my $self = shift;
  return scalar @{$self->{a16}};
}


sub a16
{
  my $self = shift;
  my $i = shift;

  $i = defined $i ? $i : 0;

  return $self->{a16}->[$i];
}


sub na4
{
  my $self = shift;
  return scalar @{$self->{a4}};
}


sub a4
{
  my $self = shift;
  my $i = shift;

  $i = defined $i ? $i : 0;

  return $self->{a4}->[$i];
}

1;
