# =============
# QPACE/TNWTEST
# =============
# $Id: tnwtest.pm 3172 2010-02-06 19:04:14Z pleiter $
#---------------------------------------------------------------------------------------------------

package testtnw;

use strict;

use lib("$ENV{QRUNDIR}/pm");
use defs qw($opt_v);
use partition;
use remote;
use io;

use lib("$ENV{TNWTESTDIR}/pm");
use globals qw($RDIR $RSH);
use globals qw($opt_cpattern $opt_dpattern $opt_dry $opt_eth);
use globals qw($opt_msgsize $opt_chmin $opt_chmax $opt_nmsg $opt_ninner);
use globals qw($opt_resetcnt $opt_timeout);


#---------------------------------------------------------------------------------------------------
# Read from remote pipe
#---------------------------------------------------------------------------------------------------

sub ncRdPipe
{
  my ($node, $cmd, $fh) = @_;

  $cmd = "$RSH root\@" . $node->hostname . " $cmd";
  io::prInfo("cmd = \"$cmd\"");

  $$fh = undef;
  if (!$node->isUp) {
    io::fprLog("Node \"%s\" down [skipped]\n", $node->hostname);
  }
  elsif (!$opt_dry) {
    $$fh = new FileHandle "$cmd |";
    die "Failed to open pipe for reading ($!)\n" unless defined $$fh;
  }
}


#---------------------------------------------------------------------------------------------------
# Killall node-card excerciser processes
#---------------------------------------------------------------------------------------------------

sub killall($$)
{
  my ($partition, @applLst) = @_;

  return if defined $opt_dry;

  foreach my $appl (@applLst) {
    remote::exec($partition, "killall $appl");
  }
}


#---------------------------------------------------------------------------------------------------
# Setup remote execution directory and files
#---------------------------------------------------------------------------------------------------

sub ncSetup($)
{
  my ($partition) = @_;

  return if defined $opt_dry;

  my $cmd = "rm -rf $RDIR; mkdir -p $RDIR/bin; mkdir -p $RDIR/log";
  remote::exec($partition, "\"$cmd\"");

  my @exeLst = grep(-x $_, grep(-f $_, glob("$ENV{TNWTESTDIR}/node/*")));
  remote::copy_to($partition, @exeLst, "$RDIR/bin");
}


#---------------------------------------------------------------------------------------------------
# Return 1 if all PPU exercisers are completed
#---------------------------------------------------------------------------------------------------

sub ppuDone($)
{
  my ($partition) = @_;

  foreach my $node (@{$$partition->{ncLst}}) {
    return 0 if ($node->{isUp} == 1 && $node->{completed} == 0);
  }

  io::prDebug("All nodes done\n");

  return 1;
}


#---------------------------------------------------------------------------------------------------
# Show PPU log messages (incremental)
#---------------------------------------------------------------------------------------------------

sub dmesgPpu($)
{
  my ($partition) = @_;

  foreach my $node (@{$$partition->{ncLst}}) {
    my $fh;
    my $cmd = "sed -n \\\'" . ($node->{nLinesRead}+1) . ",\\\$p\\\' " .
              "$RDIR/log/tnwTest.log";
    ncRdPipe($node, $cmd, \$fh);
    if (defined $fh) {
      while (<$fh>) {
        io::prLog($_);
        $node->{completed} = 1 if (/### END/);
        if (/== GLOBAL SIGNALS ==/) {
          io::fprLog("Exception detected on %s\n", $node->hostname);
          $node->{isUp} = 0;
        }
        $node->{nLinesRead}++;
      }
      $fh->close();
    }
  }
}


#---------------------------------------------------------------------------------------------------
# Show SPU log messages (all in once)
#---------------------------------------------------------------------------------------------------

sub dmesgSpu($)
{
  my ($partition) = @_;

  foreach my $node (@{$$partition->{ncLst}}) {
    my $fh;
    ncRdPipe($node, "cat $RDIR/log/lnk*.log", \$fh);
    if (defined $fh) {
      while (<$fh>) {
        io::prLog($_);
      }
      $fh->close();
    }
  }
}


#---------------------------------------------------------------------------------------------------
# Action = run
#---------------------------------------------------------------------------------------------------

sub run($)
{
  my ($partition) = @_;

  # Generate scripts
  my $wrapScript = "tnwTest-wrap-$$.sh";
  my $exeScript  = "tnwTest-$$.sh";
  io::prDebug("wrapScript = $wrapScript\n");
  io::prDebug("exeScript  = $exeScript\n");

  open(OUT, "> $wrapScript");
  print OUT "#!/bin/sh -x\n";
  print OUT "nohup $RDIR/bin/$exeScript < /dev/null >> $RDIR/log/tnwTest.log 2>&1 &\n";
  close(OUT);
  chmod 0755, $wrapScript;

  open(OUT, "> $exeScript");
  print OUT "#!/bin/sh -x\n";
  print OUT "fpga_info\n";
  print OUT "uptime\n";
  print OUT "ifdown eth0\n" if defined $opt_eth;
  if (defined $opt_resetcnt) {
    print OUT "mem_acc w64 20000512018 0000000000000000\n";
    print OUT "mem_acc w64 20000512020 0000000000000000\n";
    print OUT "dcr w gbif 0x40c 0x0\n";
    print OUT "dcr w gbif 0x410 0x0\n";
  }
  print OUT "date\n";
  print OUT "$RDIR/bin/exchange " .
            "--cpattern=$opt_cpattern --dpattern=$opt_dpattern " .
            "--verbose=$opt_v " .
            "--msgsize=$opt_msgsize " .
            "--chmin=$opt_chmin --chmax=$opt_chmax --nmsg=$opt_nmsg " .
            "--ninner=$opt_ninner";
  print OUT " --timeout=$opt_timeout" if defined $opt_timeout;
  print OUT "date\n";
  print OUT "ifup eth0\n" if defined $opt_eth;
  close(OUT);
  chmod 0755, $exeScript;

  unless (defined $opt_dry) {
    remote::copy_to($partition, "$wrapScript", "$exeScript", "$RDIR/bin");
    remote::exec($partition, "$RDIR/bin/$wrapScript");
  }

  unlink $wrapScript, $exeScript;
}


#---------------------------------------------------------------------------------------------------
# Action = stop
#---------------------------------------------------------------------------------------------------

sub stop($)
{
  my ($partition) = @_;

  return if defined $opt_dry;

  # Kill all processes
  killall($partition, "exchange");
  sleep(1) unless $opt_dry;
  killall($partition, "exchange");
}

1;
